//Copyright 2016 J.-M. CHESNEAUX, F. JEZEQUEL, J.-L. LAMOTTE
//
//This file is part of CADNA.
//
//    CADNA is free software: you can redistribute it and/or modify it
//    under the terms of the GNU Lesser General Public License as
//    published by the Free Software Foundation, either version 3 of the
//    License, or (at your option) any later version.
//
//    CADNA is distributed in the hope that it will be useful, but WITHOUT
//    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
//    Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with CADNA. If not, see
//    <http://www.gnu.org/licenses/>.
//
#ifndef __CADNA__
#define __CADNA__


#include <iostream>
#include <cmath>
#include <complex>

#define CADNA_DIV       0x1
#define CADNA_MUL       0x2
#define CADNA_POWER     0x4
#define CADNA_MATH      0x8
#define CADNA_BRANCHING 0x10
#define CADNA_INTRINSIC 0x20
#define CADNA_CANCEL    0x40
#define CADNA_ALL       0xFFFFFFFF

using namespace std;

class float_st;

class double_st 
{  
  friend class float_st;

protected:
  double x,y,z;
  
  typedef double TYPEBASE;
  
private:
  mutable int accuracy;
  int pad;

public:

  inline double_st(void) :x(0),y(0),z(0),accuracy(15) {};
  inline double_st(double a, double b, double c):x(a),y(b),z(c),accuracy(-1) {};
  inline double_st(const double &a): x(a),y(a),z(a),accuracy(15) {};

  inline double_st (const float_st&);

  double getx() {return(this->x);};
  double gety() {return(this->y);};
  double getz() {return(this->z);};


  //complex
//1)mulplication
  friend complex<double_st> operator*(double const&,complex<double_st> const&);
  friend complex<double_st> operator*(complex<double_st> const&,double const&);
  friend complex<float_st> operator*( float const&,complex<float_st> const&);
  friend complex<float_st> operator*( complex<float_st> const&,float const&);
//2)division
  friend complex<double_st> operator/(double const&,complex<double_st> const&);
  friend complex<double_st> operator/(complex<double_st> const&,double const&);
  friend complex<float_st> operator/( float const&,complex<float_st> const&);
  friend complex<float_st> operator/( complex<float_st> const&,float const&);
//3)addition
  friend complex<double_st> operator+(double const&,complex<double_st> const&);
  friend complex<double_st> operator+(complex<double_st> const&,double const&);
  friend complex<float_st> operator+( float const&,complex<float_st> const&);
  friend complex<float_st> operator+( complex<float_st> const&,float const&);
//4)SUBTRACTION
  friend complex<double_st> operator-(double const&,complex<double_st> const&);
  friend complex<double_st> operator-(complex<double_st> const&,double const&);
  friend complex<float_st> operator-( float const&,complex<float_st> const&);
  friend complex<float_st> operator-( complex<float_st> const&,float const&);

  // ASSIGNMENT
  
  double_st& operator=(const double&);
  double_st& operator=(const float&);
  double_st& operator=(const unsigned long long&);
  double_st& operator=(const long long&);
  double_st& operator=(const unsigned long&);
  double_st& operator=(const long&);
  double_st& operator=(const unsigned int&);
  double_st& operator=(const int&);
  double_st& operator=(const unsigned short&);
  double_st& operator=(const short&);
  double_st& operator=(const unsigned char&);
  double_st& operator=(const char&);

  double_st& operator=(const float_st&);


  // ADDITION
  double_st operator++();
  double_st operator++(int);
  double_st operator+() const ; 

  double_st& operator+=(const double_st&);
  double_st& operator+=(const float_st&);
  double_st& operator+=(const double&);
  double_st& operator+=(const float&);
  double_st& operator+=(const unsigned long long&);
  double_st& operator+=(const long long&);
  double_st& operator+=(const unsigned long&);
  double_st& operator+=(const long&);
  double_st& operator+=(const unsigned int&);
  double_st& operator+=(const int&);
  double_st& operator+=(const unsigned short&);
  double_st& operator+=(const short&);
  double_st& operator+=(const unsigned char&);
  double_st& operator+=(const char&);


  friend double_st operator+(const double_st&, const double_st&);  
  friend double_st operator+(const double_st&, const float_st&);  
  friend double_st operator+(const float_st&,  const double_st&);
  
  friend double_st operator+(const double_st&, const double&);
  friend double_st operator+(const double_st&, const float&);
  friend double_st operator+(const double_st&, const unsigned long long&);
  friend double_st operator+(const double_st&, const long long&);
  friend double_st operator+(const double_st&, const unsigned long&);
  friend double_st operator+(const double_st&, const long&);
  friend double_st operator+(const double_st&, const unsigned int&);
  friend double_st operator+(const double_st&, const int&);
  friend double_st operator+(const double_st&, const unsigned short&);
  friend double_st operator+(const double_st&, const short&);
  friend double_st operator+(const double_st&, const unsigned char&);
  friend double_st operator+(const double_st&, const char&);

  friend double_st operator+(const double&, const double_st&);
  friend double_st operator+(const float&, const double_st&);
  friend double_st operator+(const unsigned long long&, const double_st&);
  friend double_st operator+(const long long&, const double_st&);
  friend double_st operator+(const unsigned long&, const double_st&);
  friend double_st operator+(const long&, const double_st&);
  friend double_st operator+(const unsigned int&, const double_st&);
  friend double_st operator+(const int&, const double_st&);
  friend double_st operator+(const unsigned short&, const double_st&);
  friend double_st operator+(const short&, const double_st&);
  friend double_st operator+(const unsigned char&, const double_st&);
  friend double_st operator+(const char&, const double_st&);


      
  // SUBTRACTION
  double_st operator--();
  double_st operator--(int);
  double_st operator-() const ; 

  double_st& operator-=(const double_st&);
  double_st& operator-=(const float_st&);
  double_st& operator-=(const double&);
  double_st& operator-=(const float&);
  double_st& operator-=(const unsigned long long&);
  double_st& operator-=(const long long&);
  double_st& operator-=(const unsigned long&);
  double_st& operator-=(const long&);
  double_st& operator-=(const unsigned int&);
  double_st& operator-=(const int&);
  double_st& operator-=(const unsigned short&);
  double_st& operator-=(const short&);
  double_st& operator-=(const unsigned char&);
  double_st& operator-=(const char&);


  friend double_st operator-(const double_st&, const double_st&);  
  friend double_st operator-(const double_st&, const float_st&);  
  friend double_st operator-(const float_st&, const double_st&);  
  friend double_st operator-(const double_st&, const double&);
  friend double_st operator-(const double_st&, const float&);
  friend double_st operator-(const double_st&, const unsigned long long&);
  friend double_st operator-(const double_st&, const long long&);
  friend double_st operator-(const double_st&, const unsigned long&);
  friend double_st operator-(const double_st&, const long&);
  friend double_st operator-(const double_st&, const unsigned int&);
  friend double_st operator-(const double_st&, const int&);
  friend double_st operator-(const double_st&, const unsigned short&);
  friend double_st operator-(const double_st&, const short&);
  friend double_st operator-(const double_st&, const unsigned char&);
  friend double_st operator-(const double_st&, const char&);

  friend double_st operator-(const double&, const double_st&);
  friend double_st operator-(const float&, const double_st&);
  friend double_st operator-(const unsigned long long&, const double_st&);
  friend double_st operator-(const long long&, const double_st&);
  friend double_st operator-(const unsigned long&, const double_st&);
  friend double_st operator-(const long&, const double_st&);
  friend double_st operator-(const unsigned int&, const double_st&);
  friend double_st operator-(const int&, const double_st&);
  friend double_st operator-(const unsigned short&, const double_st&);
  friend double_st operator-(const short&, const double_st&);
  friend double_st operator-(const unsigned char&, const double_st&);
  friend double_st operator-(const char&, const double_st&);

  // MULTIPLICATION

  double_st& operator*=(const double_st&);
  double_st& operator*=(const float_st&);
  double_st& operator*=(const double&);
  double_st& operator*=(const float&);
  double_st& operator*=(const unsigned long long&);
  double_st& operator*=(const long long&);
  double_st& operator*=(const unsigned long&);
  double_st& operator*=(const long&);
  double_st& operator*=(const unsigned int&);
  double_st& operator*=(const int&);
  double_st& operator*=(const unsigned short&);
  double_st& operator*=(const short&);
  double_st& operator*=(const unsigned char&);
  double_st& operator*=(const char&);


  friend double_st operator*(const double_st&, const double_st&);  
  friend double_st operator*(const double_st&, const float_st&);  
  friend double_st operator*(const float_st&, const double_st&);  

  friend double_st operator*(const double_st&, const double&);
  friend double_st operator*(const double_st&, const float&);
  friend double_st operator*(const double_st&, const unsigned long long&);
  friend double_st operator*(const double_st&, const long long&);
  friend double_st operator*(const double_st&, const unsigned long&);
  friend double_st operator*(const double_st&, const long&);
  friend double_st operator*(const double_st&, const unsigned int&);
  friend double_st operator*(const double_st&, const int&);
  friend double_st operator*(const double_st&, const unsigned short&);
  friend double_st operator*(const double_st&, const short&);
  friend double_st operator*(const double_st&, const unsigned char&);
  friend double_st operator*(const double_st&, const char&);

  friend double_st operator*(const double&, const double_st&);
  friend double_st operator*(const float&, const double_st&);
  friend double_st operator*(const unsigned long long&, const double_st&);
  friend double_st operator*(const long long&, const double_st&);
  friend double_st operator*(const unsigned long&, const double_st&);
  friend double_st operator*(const long&, const double_st&);
  friend double_st operator*(const unsigned int&, const double_st&);
  friend double_st operator*(const int&, const double_st&);
  friend double_st operator*(const unsigned short&, const double_st&);
  friend double_st operator*(const short&, const double_st&);
  friend double_st operator*(const unsigned char&, const double_st&);
  friend double_st operator*(const char&, const double_st&);

  // DIVISION

  double_st& operator/=(const double_st&);
  double_st& operator/=(const float_st&);
  double_st& operator/=(const double&);
  double_st& operator/=(const float&);
  double_st& operator/=(const unsigned long long&);
  double_st& operator/=(const long long&);
  double_st& operator/=(const unsigned long&);
  double_st& operator/=(const long&);
  double_st& operator/=(const unsigned int&);
  double_st& operator/=(const int&);
  double_st& operator/=(const unsigned short&);
  double_st& operator/=(const short&);
  double_st& operator/=(const unsigned char&);
  double_st& operator/=(const char&);


  friend double_st operator/(const double_st&, const double_st&);  
  friend double_st operator/(const double_st&, const float_st&);  
  friend double_st operator/(const float_st&, const double_st&);  
  friend double_st operator/(const double_st&, const double&);
  friend double_st operator/(const double_st&, const float&);
  friend double_st operator/(const double_st&, const unsigned long long&);
  friend double_st operator/(const double_st&, const long long&);
  friend double_st operator/(const double_st&, const unsigned long&);
  friend double_st operator/(const double_st&, const long&);
  friend double_st operator/(const double_st&, const unsigned int&);
  friend double_st operator/(const double_st&, const int&);
  friend double_st operator/(const double_st&, const unsigned short&);
  friend double_st operator/(const double_st&, const short&);
  friend double_st operator/(const double_st&, const unsigned char&);
  friend double_st operator/(const double_st&, const char&);

  friend double_st operator/(const double&, const double_st&);
  friend double_st operator/(const float&, const double_st&);
  friend double_st operator/(const unsigned long long&, const double_st&);
  friend double_st operator/(const long long&, const double_st&);
  friend double_st operator/(const unsigned long&, const double_st&);
  friend double_st operator/(const long&, const double_st&);
  friend double_st operator/(const unsigned int&, const double_st&);
  friend double_st operator/(const int&, const double_st&);
  friend double_st operator/(const unsigned short&, const double_st&);
  friend double_st operator/(const short&, const double_st&);
  friend double_st operator/(const unsigned char&, const double_st&);
  friend double_st operator/(const char&, const double_st&);
  
  
  // RELATIONAL OPERATORS

  friend int operator==(const double_st&, const double_st&);  
  friend int operator==(const double_st&, const float_st&);  
  friend int operator==(const float_st&, const double_st&);  

  friend int operator==(const double_st&, const double&);
  friend int operator==(const double_st&, const float&);
  friend int operator==(const double_st&, const unsigned long long&);
  friend int operator==(const double_st&, const long long&);
  friend int operator==(const double_st&, const unsigned long&);
  friend int operator==(const double_st&, const long&);
  friend int operator==(const double_st&, const unsigned int&);
  friend int operator==(const double_st&, const int&);
  friend int operator==(const double_st&, const unsigned short&);
  friend int operator==(const double_st&, const short&);
  friend int operator==(const double_st&, const unsigned char&);
  friend int operator==(const double_st&, const char&);

  friend int operator==(const double&, const double_st&);
  friend int operator==(const float&, const double_st&);
  friend int operator==(const unsigned long long&, const double_st&);
  friend int operator==(const long long&, const double_st&);
  friend int operator==(const unsigned long&, const double_st&);
  friend int operator==(const long&, const double_st&);
  friend int operator==(const unsigned int&, const double_st&);
  friend int operator==(const int&, const double_st&);
  friend int operator==(const unsigned short&, const double_st&);
  friend int operator==(const short&, const double_st&);
  friend int operator==(const unsigned char&, const double_st&);
  friend int operator==(const char&, const double_st&);
  


  friend int operator!=(const double_st&, const double_st&);  
  friend int operator!=(const double_st&, const float_st&);  
  friend int operator!=(const float_st&, const double_st&);  

  friend int operator!=(const double_st&, const double&);
  friend int operator!=(const double_st&, const float&);
  friend int operator!=(const double_st&, const unsigned long long&);
  friend int operator!=(const double_st&, const long long&);
  friend int operator!=(const double_st&, const unsigned long&);
  friend int operator!=(const double_st&, const long&);
  friend int operator!=(const double_st&, const unsigned int&);
  friend int operator!=(const double_st&, const int&);
  friend int operator!=(const double_st&, const unsigned short&);
  friend int operator!=(const double_st&, const short&);
  friend int operator!=(const double_st&, const unsigned char&);
  friend int operator!=(const double_st&, const char&);

  friend int operator!=(const double&, const double_st&);
  friend int operator!=(const float&, const double_st&);
  friend int operator!=(const unsigned long long&, const double_st&);
  friend int operator!=(const long long&, const double_st&);
  friend int operator!=(const unsigned long&, const double_st&);
  friend int operator!=(const long&, const double_st&);
  friend int operator!=(const unsigned int&, const double_st&);
  friend int operator!=(const int&, const double_st&);
  friend int operator!=(const unsigned short&, const double_st&);
  friend int operator!=(const short&, const double_st&);
  friend int operator!=(const unsigned char&, const double_st&);
  friend int operator!=(const char&, const double_st&);

  friend int operator>=(const double_st&, const double_st&);  
  friend int operator>=(const double_st&, const float_st&);  
  friend int operator>=(const float_st&, const double_st&);  

  friend int operator>=(const double_st&, const double&);
  friend int operator>=(const double_st&, const float&);
  friend int operator>=(const double_st&, const unsigned long long&);
  friend int operator>=(const double_st&, const long long&);
  friend int operator>=(const double_st&, const unsigned long&);
  friend int operator>=(const double_st&, const long&);
  friend int operator>=(const double_st&, const unsigned int&);
  friend int operator>=(const double_st&, const int&);
  friend int operator>=(const double_st&, const unsigned short&);
  friend int operator>=(const double_st&, const short&);
  friend int operator>=(const double_st&, const unsigned char&);
  friend int operator>=(const double_st&, const char&);

  friend int operator>=(const double&, const double_st&);
  friend int operator>=(const float&, const double_st&);
  friend int operator>=(const unsigned long long&, const double_st&);
  friend int operator>=(const long long&, const double_st&);
  friend int operator>=(const unsigned long&, const double_st&);
  friend int operator>=(const long&, const double_st&);
  friend int operator>=(const unsigned int&, const double_st&);
  friend int operator>=(const int&, const double_st&);
  friend int operator>=(const unsigned short&, const double_st&);
  friend int operator>=(const short&, const double_st&);
  friend int operator>=(const unsigned char&, const double_st&);
  friend int operator>=(const char&, const double_st&);

  friend int operator>(const double_st&, const double_st&);  
  friend int operator>(const double_st&, const float_st&);  
  friend int operator>(const float_st&, const double_st&);  

  friend int operator>(const double_st&, const double&);
  friend int operator>(const double_st&, const float&);
  friend int operator>(const double_st&, const unsigned long long&);
  friend int operator>(const double_st&, const long long&);
  friend int operator>(const double_st&, const unsigned long&);
  friend int operator>(const double_st&, const long&);
  friend int operator>(const double_st&, const unsigned int&);
  friend int operator>(const double_st&, const int&);
  friend int operator>(const double_st&, const unsigned short&);
  friend int operator>(const double_st&, const short&);
  friend int operator>(const double_st&, const unsigned char&);
  friend int operator>(const double_st&, const char&);

  friend int operator>(const double&, const double_st&);
  friend int operator>(const float&, const double_st&);
  friend int operator>(const unsigned long long&, const double_st&);
  friend int operator>(const long long&, const double_st&);
  friend int operator>(const unsigned long&, const double_st&);
  friend int operator>(const long&, const double_st&);
  friend int operator>(const unsigned int&, const double_st&);
  friend int operator>(const int&, const double_st&);
  friend int operator>(const unsigned short&, const double_st&);
  friend int operator>(const short&, const double_st&);
  friend int operator>(const unsigned char&, const double_st&);
  friend int operator>(const char&, const double_st&);

  friend int operator<=(const double_st&, const double_st&);  
  friend int operator<=(const double_st&, const float_st&);  
  friend int operator<=(const float_st&, const double_st&);  

  friend int operator<=(const double_st&, const double&);
  friend int operator<=(const double_st&, const float&);
  friend int operator<=(const double_st&, const unsigned long long&);
  friend int operator<=(const double_st&, const long long&);
  friend int operator<=(const double_st&, const unsigned long&);
  friend int operator<=(const double_st&, const long&);
  friend int operator<=(const double_st&, const unsigned int&);
  friend int operator<=(const double_st&, const int&);
  friend int operator<=(const double_st&, const unsigned short&);
  friend int operator<=(const double_st&, const short&);
  friend int operator<=(const double_st&, const unsigned char&);
  friend int operator<=(const double_st&, const char&);

  friend int operator<=(const double&, const double_st&);
  friend int operator<=(const float&, const double_st&);
  friend int operator<=(const unsigned long long&, const double_st&);
  friend int operator<=(const long long&, const double_st&);
  friend int operator<=(const unsigned long&, const double_st&);
  friend int operator<=(const long&, const double_st&);
  friend int operator<=(const unsigned int&, const double_st&);
  friend int operator<=(const int&, const double_st&);
  friend int operator<=(const unsigned short&, const double_st&);
  friend int operator<=(const short&, const double_st&);
  friend int operator<=(const unsigned char&, const double_st&);
  friend int operator<=(const char&, const double_st&);

  friend int operator<(const double_st&, const double_st&);  
  friend int operator<(const double_st&, const float_st&);  
  friend int operator<(const float_st&, const double_st&);  

  friend int operator<(const double_st&, const double&);
  friend int operator<(const double_st&, const float&);
  friend int operator<(const double_st&, const unsigned long long&);
  friend int operator<(const double_st&, const long long&);
  friend int operator<(const double_st&, const unsigned long&);
  friend int operator<(const double_st&, const long&);
  friend int operator<(const double_st&, const unsigned int&);
  friend int operator<(const double_st&, const int&);
  friend int operator<(const double_st&, const unsigned short&);
  friend int operator<(const double_st&, const short&);
  friend int operator<(const double_st&, const unsigned char&);
  friend int operator<(const double_st&, const char&);

  friend int operator<(const double&, const double_st&);
  friend int operator<(const float&, const double_st&);
  friend int operator<(const unsigned long long&, const double_st&);
  friend int operator<(const long long&, const double_st&);
  friend int operator<(const unsigned long&, const double_st&);
  friend int operator<(const long&, const double_st&);
  friend int operator<(const unsigned int&, const double_st&);
  friend int operator<(const int&, const double_st&);
  friend int operator<(const unsigned short&, const double_st&);
  friend int operator<(const short&, const double_st&);
  friend int operator<(const unsigned char&, const double_st&);
  friend int operator<(const char&, const double_st&);


  // MATHEMATICAL FUNCTIONS 

  friend double_st  log(const double_st&);
  friend double_st  log2(const double_st&);
  friend double_st  log10(const double_st&);
  friend double_st  log1p(const double_st&);
  friend double_st  logb(const double_st&);
  friend double_st  exp(const double_st&); 
  friend double_st  exp2(const double_st&); 
  friend double_st  expm1(const double_st&); 
  friend double_st  sqrt(const double_st&);
  friend double_st  cbrt(const double_st&);
  friend double_st  sin(const double_st&);
  friend double_st  cos(const double_st&);
  friend double_st  tan(const double_st&);
  friend double_st  asin(const double_st&);
  friend double_st  acos(const double_st&);
  friend double_st  atan(const double_st&);
  friend double_st  atan2(const double_st&, const double_st&);
  friend double_st  atan2(const double_st&, const float_st&);
  friend double_st  atan2(const float_st&, const double_st&);
  friend double_st  sinh(const double_st&);
  friend double_st  cosh(const double_st&);
  friend double_st  tanh(const double_st&);
  friend double_st  asinh(const double_st&);
  friend double_st  acosh(const double_st&);
  friend double_st  atanh(const double_st&);
  friend double_st  hypot(const double_st&, const double_st&);
  friend double_st  hypot(const double_st&, const float_st&);
  friend double_st  hypot(const float_st&, const double_st&);

  friend double_st pow(const double_st&, const double_st&);  
  friend double_st pow(const double_st&, const float_st&);  
  friend double_st pow(const float_st&, const double_st&);  
  friend double_st pow(const double_st&, const double&);
  friend double_st pow(const double_st&, const float&);
  friend double_st pow(const double_st&, const unsigned long long&);
  friend double_st pow(const double_st&, const long long&);
  friend double_st pow(const double_st&, const unsigned long&);
  friend double_st pow(const double_st&, const long&);
  friend double_st pow(const double_st&, const unsigned int&);
  friend double_st pow(const double_st&, const int&);
  friend double_st pow(const double_st&, const unsigned short&);
  friend double_st pow(const double_st&, const short&);
  friend double_st pow(const double_st&, const unsigned char&);
  friend double_st pow(const double_st&, const char&);

  friend double_st pow(const double&, const double_st&);
  friend double_st pow(const float&, const double_st&);
  friend double_st pow(const unsigned long long&, const double_st&);
  friend double_st pow(const long long&, const double_st&);
  friend double_st pow(const unsigned long&, const double_st&);
  friend double_st pow(const long&, const double_st&);
  friend double_st pow(const unsigned int&, const double_st&);
  friend double_st pow(const int&, const double_st&);
  friend double_st pow(const unsigned short&, const double_st&);
  friend double_st pow(const short&, const double_st&);
  friend double_st pow(const unsigned char&, const double_st&);
  friend double_st pow(const char&, const double_st&);


  friend double_st fmax(const double_st&, const double_st&);  
  friend double_st fmax(const double_st&, const float_st&);  
  friend double_st fmax(const float_st&, const double_st&);  
  friend double_st fmax(const double_st&, const double&);
  friend double_st fmax(const double_st&, const float&);
  friend double_st fmax(const double_st&, const unsigned long long&);
  friend double_st fmax(const double_st&, const long long&);
  friend double_st fmax(const double_st&, const unsigned long&);
  friend double_st fmax(const double_st&, const long&);
  friend double_st fmax(const double_st&, const unsigned int&);
  friend double_st fmax(const double_st&, const int&);
  friend double_st fmax(const double_st&, const unsigned short&);
  friend double_st fmax(const double_st&, const short&);
  friend double_st fmax(const double_st&, const unsigned char&);
  friend double_st fmax(const double_st&, const char&);

  friend double_st fmax(const double&, const double_st&);
  friend double_st fmax(const float&, const double_st&);
  friend double_st fmax(const unsigned long long&, const double_st&);
  friend double_st fmax(const long long&, const double_st&);
  friend double_st fmax(const unsigned long&, const double_st&);
  friend double_st fmax(const long&, const double_st&);
  friend double_st fmax(const unsigned int&, const double_st&);
  friend double_st fmax(const int&, const double_st&);
  friend double_st fmax(const unsigned short&, const double_st&);
  friend double_st fmax(const short&, const double_st&);
  friend double_st fmax(const unsigned char&, const double_st&);
  friend double_st fmax(const char&, const double_st&);


  friend double_st fmin(const double_st&, const double_st&);  
  friend double_st fmin(const double_st&, const float_st&);  
  friend double_st fmin(const float_st&, const double_st&);  
  
  friend double_st fmin(const double_st&, const double&);
  friend double_st fmin(const double_st&, const float&);
  friend double_st fmin(const double_st&, const unsigned long long&);
  friend double_st fmin(const double_st&, const long long&);
  friend double_st fmin(const double_st&, const unsigned long&);
  friend double_st fmin(const double_st&, const long&);
  friend double_st fmin(const double_st&, const unsigned int&);
  friend double_st fmin(const double_st&, const int&);
  friend double_st fmin(const double_st&, const unsigned short&);
  friend double_st fmin(const double_st&, const short&);
  friend double_st fmin(const double_st&, const unsigned char&);
  friend double_st fmin(const double_st&, const char&);

  friend double_st fmin(const double&, const double_st&);
  friend double_st fmin(const float&, const double_st&);
  friend double_st fmin(const unsigned long long&, const double_st&);
  friend double_st fmin(const long long&, const double_st&);
  friend double_st fmin(const unsigned long&, const double_st&);
  friend double_st fmin(const long&, const double_st&);
  friend double_st fmin(const unsigned int&, const double_st&);
  friend double_st fmin(const int&, const double_st&);
  friend double_st fmin(const unsigned short&, const double_st&);
  friend double_st fmin(const short&, const double_st&);
  friend double_st fmin(const unsigned char&, const double_st&);
  friend double_st fmin(const char&, const double_st&);

  friend double_st frexp(const double_st&, int* );
  friend double_st modf(const  double_st&,  double_st*);
  friend double_st ldexp(const double_st&, const int);
  friend double_st fmod(const double_st&, const double_st&);

  // INTRINSIC FUNCTIONS
  
  friend double_st fabs(const double_st&);
  friend double_st abs(const double_st&);
  friend double_st floor(const double_st&);
  friend double_st ceil(const double_st&);
  friend double_st trunc(const double_st&);
  friend double_st nearbyint(const double_st&);
  friend double_st rint(const double_st&);
  friend long int  lrint(const double_st&);
  friend long long int llrint(const double_st&);

  friend int finite(const double_st&);
  friend int isfinite(const double_st&);
  friend int isnan(const double_st&);
  friend int isinf(const double_st&);


  // Conversion
  operator char();  
  operator unsigned char();  
  operator short();  
  operator unsigned short();  
  operator int();  
  operator unsigned int();  
  operator long();  
  operator unsigned long();  
  operator long long();  
  operator unsigned long long();  
  operator float();
  operator double();
  
  int nb_significant_digit() const;  
  int approx_digit() const;  
  int computedzero() const;
  int numericalnoise() const;
  void display() const ; 
  void display(const char *) const ; 
  char* str( char *)  const ;
  
  friend char* strp(const double_st&);
  friend char* str( char *, const double_st&);
  
  friend std::istream& operator >>(std::istream& s, double_st& );  
  
  void data_st();
  void data_st(const double&, const int&);
  
};

std::ostream& operator<<(std::ostream&, const double_st&);
std::istream& operator >>(std::istream&, double_st& );

///////////////////////////////
///////////////////////////////
///////////////////////////////

class float_st {
  friend class double_st;

protected:
  float x,y,z;
  
  typedef float TYPEBASE;
  
private:
  mutable int accuracy;
  
public:

  inline float_st(void) :x(0),y(0),z(0),accuracy(7) {};
  inline float_st(const double &a, const double &b, const double &c):x(a),y(b),z(c),accuracy(-1) {};

  inline float_st(const double &a): x(a),y(a),z(a),accuracy(7) {};
  inline float_st (const double_st&a) : x(a.x),y(a.y),z(a.z),accuracy(a.accuracy>8?7:a.accuracy) {};

  float getx() {return(this->x);};
  float gety() {return(this->y);};
  float getz() {return(this->z);};


  // ASSIGNMENT

  //  float_st& operator=(const double_st&);
  float_st& operator=(const float_st&); 
  float_st& operator=(const double&);
  float_st& operator=(const float&);
  float_st& operator=(const unsigned long long&);
  float_st& operator=(const long long&);
  float_st& operator=(const unsigned long&);
  float_st& operator=(const long&);
  float_st& operator=(const unsigned int&);
  float_st& operator=(const int&);
  float_st& operator=(const unsigned short&);
  float_st& operator=(const short&);
  float_st& operator=(const unsigned char&);
  float_st& operator=(const char&);
  
  float_st& operator=(const double_st&);


  // ADDITION
  float_st operator++();
  float_st operator++(int);
  float_st operator+() const ; 

  float_st& operator+=(const double_st&);
  float_st& operator+=(const float_st&);
  float_st& operator+=(const double&);
  float_st& operator+=(const float&);
  float_st& operator+=(const unsigned long long&);
  float_st& operator+=(const long long&);
  float_st& operator+=(const unsigned long&);
  float_st& operator+=(const long&);
  float_st& operator+=(const unsigned int&);
  float_st& operator+=(const int&);
  float_st& operator+=(const unsigned short&);
  float_st& operator+=(const short&);
  float_st& operator+=(const unsigned char&);
  float_st& operator+=(const char&);

  friend double_st operator+(const double_st&, const float_st&);  
  friend double_st operator+(const float_st&,  const double_st&);  
  friend float_st operator+(const float_st&, const float_st&);  

  friend float_st operator+(const float_st&, const double&);
  friend float_st operator+(const float_st&, const float&);
  friend float_st operator+(const float_st&, const unsigned long long&);
  friend float_st operator+(const float_st&, const long long&);
  friend float_st operator+(const float_st&, const unsigned long&);
  friend float_st operator+(const float_st&, const long&);
  friend float_st operator+(const float_st&, const unsigned int&);
  friend float_st operator+(const float_st&, const int&);
  friend float_st operator+(const float_st&, const unsigned short&);
  friend float_st operator+(const float_st&, const short&);
  friend float_st operator+(const float_st&, const unsigned char&);
  friend float_st operator+(const float_st&, const char&);

  friend float_st operator+(const double&, const float_st&);
  friend float_st operator+(const float&, const float_st&);
  friend float_st operator+(const unsigned long long&, const float_st&);
  friend float_st operator+(const long long&, const float_st&);
  friend float_st operator+(const unsigned long&, const float_st&);
  friend float_st operator+(const long&, const float_st&);
  friend float_st operator+(const unsigned int&, const float_st&);
  friend float_st operator+(const int&, const float_st&);
  friend float_st operator+(const unsigned short&, const float_st&);
  friend float_st operator+(const short&, const float_st&);
  friend float_st operator+(const unsigned char&, const float_st&);
  friend float_st operator+(const char&, const float_st&);

      
  // SUBTRACTION

  float_st operator--();
  float_st operator--(int);
  float_st operator-() const ; 

  float_st& operator-=(const double_st&);
  float_st& operator-=(const float_st&);
  float_st& operator-=(const double&);
  float_st& operator-=(const float&);
  float_st& operator-=(const unsigned long long&);
  float_st& operator-=(const long long&);
  float_st& operator-=(const unsigned long&);
  float_st& operator-=(const long&);
  float_st& operator-=(const unsigned int&);
  float_st& operator-=(const int&);
  float_st& operator-=(const unsigned short&);
  float_st& operator-=(const short&);
  float_st& operator-=(const unsigned char&);
  float_st& operator-=(const char&);

  friend double_st operator-(const double_st&, const float_st&);  
  friend double_st operator-(const float_st&, const double_st&);  
  friend float_st operator-(const float_st&, const float_st&);  

  friend float_st operator-(const float_st&, const double&);
  friend float_st operator-(const float_st&, const float&);
  friend float_st operator-(const float_st&, const unsigned long long&);
  friend float_st operator-(const float_st&, const long long&);
  friend float_st operator-(const float_st&, const unsigned long&);
  friend float_st operator-(const float_st&, const long&);
  friend float_st operator-(const float_st&, const unsigned int&);
  friend float_st operator-(const float_st&, const int&);
  friend float_st operator-(const float_st&, const unsigned short&);
  friend float_st operator-(const float_st&, const short&);
  friend float_st operator-(const float_st&, const unsigned char&);
  friend float_st operator-(const float_st&, const char&);

  friend float_st operator-(const double&, const float_st&);
  friend float_st operator-(const float&, const float_st&);
  friend float_st operator-(const unsigned long long&, const float_st&);
  friend float_st operator-(const long long&, const float_st&);
  friend float_st operator-(const unsigned long&, const float_st&);
  friend float_st operator-(const long&, const float_st&);
  friend float_st operator-(const unsigned int&, const float_st&);
  friend float_st operator-(const int&, const float_st&);
  friend float_st operator-(const unsigned short&, const float_st&);
  friend float_st operator-(const short&, const float_st&);
  friend float_st operator-(const unsigned char&, const float_st&);
  friend float_st operator-(const char&, const float_st&);
    
  // PRODUCT

  float_st& operator*=(const double_st&);
  float_st& operator*=(const float_st&);
  float_st& operator*=(const double&);
  float_st& operator*=(const float&);
  float_st& operator*=(const unsigned long long&);
  float_st& operator*=(const long long&);
  float_st& operator*=(const unsigned long&);
  float_st& operator*=(const long&);
  float_st& operator*=(const unsigned int&);
  float_st& operator*=(const int&);
  float_st& operator*=(const unsigned short&);
  float_st& operator*=(const short&);
  float_st& operator*=(const unsigned char&);
  float_st& operator*=(const char&);

  friend double_st operator*(const double_st&, const float_st&);  
  friend double_st operator*(const float_st&, const double_st&);  
  friend float_st operator*(const float_st&, const float_st&);  

  friend float_st operator*(const float_st&, const double&);
  friend float_st operator*(const float_st&, const float&);
  friend float_st operator*(const float_st&, const unsigned long long&);
  friend float_st operator*(const float_st&, const long long&);
  friend float_st operator*(const float_st&, const unsigned long&);
  friend float_st operator*(const float_st&, const long&);
  friend float_st operator*(const float_st&, const unsigned int&);
  friend float_st operator*(const float_st&, const int&);
  friend float_st operator*(const float_st&, const unsigned short&);
  friend float_st operator*(const float_st&, const short&);
  friend float_st operator*(const float_st&, const unsigned char&);
  friend float_st operator*(const float_st&, const char&);

  friend float_st operator*(const double&, const float_st&);
  friend float_st operator*(const float&, const float_st&);
  friend float_st operator*(const unsigned long long&, const float_st&);
  friend float_st operator*(const long long&, const float_st&);
  friend float_st operator*(const unsigned long&, const float_st&);
  friend float_st operator*(const long&, const float_st&);
  friend float_st operator*(const unsigned int&, const float_st&);
  friend float_st operator*(const int&, const float_st&);
  friend float_st operator*(const unsigned short&, const float_st&);
  friend float_st operator*(const short&, const float_st&);
  friend float_st operator*(const unsigned char&, const float_st&);
  friend float_st operator*(const char&, const float_st&);

  // DIVISION
  
  float_st& operator/=(const double_st&);
  float_st& operator/=(const float_st&);
  float_st& operator/=(const double&);
  float_st& operator/=(const float&);
  float_st& operator/=(const unsigned long long&);
  float_st& operator/=(const long long&);
  float_st& operator/=(const unsigned long&);
  float_st& operator/=(const long&);
  float_st& operator/=(const unsigned int&);
  float_st& operator/=(const int&);
  float_st& operator/=(const unsigned short&);
  float_st& operator/=(const short&);
  float_st& operator/=(const unsigned char&);
  float_st& operator/=(const char&);

  friend double_st operator/(const double_st&, const float_st&);  
  friend double_st operator/(const float_st&, const double_st&);  
  friend float_st operator/(const float_st&, const float_st&);  

  friend float_st operator/(const float_st&, const double&);
  friend float_st operator/(const float_st&, const float&);
  friend float_st operator/(const float_st&, const unsigned long long&);
  friend float_st operator/(const float_st&, const long long&);
  friend float_st operator/(const float_st&, const unsigned long&);
  friend float_st operator/(const float_st&, const long&);
  friend float_st operator/(const float_st&, const unsigned int&);
  friend float_st operator/(const float_st&, const int&);
  friend float_st operator/(const float_st&, const unsigned short&);
  friend float_st operator/(const float_st&, const short&);
  friend float_st operator/(const float_st&, const unsigned char&);
  friend float_st operator/(const float_st&, const char&);

  friend float_st operator/(const double&, const float_st&);
  friend float_st operator/(const float&, const float_st&);
  friend float_st operator/(const unsigned long long&, const float_st&);
  friend float_st operator/(const long long&, const float_st&);
  friend float_st operator/(const unsigned long&, const float_st&);
  friend float_st operator/(const long&, const float_st&);
  friend float_st operator/(const unsigned int&, const float_st&);
  friend float_st operator/(const int&, const float_st&);
  friend float_st operator/(const unsigned short&, const float_st&);
  friend float_st operator/(const short&, const float_st&);
  friend float_st operator/(const unsigned char&, const float_st&);
  friend float_st operator/(const char&, const float_st&);


  // RELATIONAL OPERATORS

  friend int operator==(const double_st&, const float_st&);  
  friend int operator==(const float_st&, const double_st&);  
  friend int operator==(const float_st&, const float_st&);  

  friend int operator==(const float_st&, const double&);
  friend int operator==(const float_st&, const float&);
  friend int operator==(const float_st&, const unsigned long long&);
  friend int operator==(const float_st&, const long long&);
  friend int operator==(const float_st&, const unsigned long&);
  friend int operator==(const float_st&, const long&);
  friend int operator==(const float_st&, const unsigned int&);
  friend int operator==(const float_st&, const int&);
  friend int operator==(const float_st&, const unsigned short&);
  friend int operator==(const float_st&, const short&);
  friend int operator==(const float_st&, const unsigned char&);
  friend int operator==(const float_st&, const char&);

  friend int operator==(const double&, const float_st&);
  friend int operator==(const float&, const float_st&);
  friend int operator==(const unsigned long long&, const float_st&);
  friend int operator==(const long long&, const float_st&);
  friend int operator==(const unsigned long&, const float_st&);
  friend int operator==(const long&, const float_st&);
  friend int operator==(const unsigned int&, const float_st&);
  friend int operator==(const int&, const float_st&);
  friend int operator==(const unsigned short&, const float_st&);
  friend int operator==(const short&, const float_st&);
  friend int operator==(const unsigned char&, const float_st&);
  friend int operator==(const char&, const float_st&);
  
  friend int operator!=(const double_st&, const float_st&);  
  friend int operator!=(const float_st&, const double_st&);  
  friend int operator!=(const float_st&, const float_st&);  

  friend int operator!=(const float_st&, const double&);
  friend int operator!=(const float_st&, const float&);
  friend int operator!=(const float_st&, const unsigned long long&);
  friend int operator!=(const float_st&, const long long&);
  friend int operator!=(const float_st&, const unsigned long&);
  friend int operator!=(const float_st&, const long&);
  friend int operator!=(const float_st&, const unsigned int&);
  friend int operator!=(const float_st&, const int&);
  friend int operator!=(const float_st&, const unsigned short&);
  friend int operator!=(const float_st&, const short&);
  friend int operator!=(const float_st&, const unsigned char&);
  friend int operator!=(const float_st&, const char&);

  friend int operator!=(const double&, const float_st&);
  friend int operator!=(const float&, const float_st&);
  friend int operator!=(const unsigned long long&, const float_st&);
  friend int operator!=(const long long&, const float_st&);
  friend int operator!=(const unsigned long&, const float_st&);
  friend int operator!=(const long&, const float_st&);
  friend int operator!=(const unsigned int&, const float_st&);
  friend int operator!=(const int&, const float_st&);
  friend int operator!=(const unsigned short&, const float_st&);
  friend int operator!=(const short&, const float_st&);
  friend int operator!=(const unsigned char&, const float_st&);
  friend int operator!=(const char&, const float_st&);

  friend int operator>=(const double_st&, const float_st&);  
  friend int operator>=(const float_st&, const double_st&);  
  friend int operator>=(const float_st&, const float_st&);  

  friend int operator>=(const float_st&, const double&);
  friend int operator>=(const float_st&, const float&);
  friend int operator>=(const float_st&, const unsigned long long&);
  friend int operator>=(const float_st&, const long long&);
  friend int operator>=(const float_st&, const unsigned long&);
  friend int operator>=(const float_st&, const long&);
  friend int operator>=(const float_st&, const unsigned int&);
  friend int operator>=(const float_st&, const int&);
  friend int operator>=(const float_st&, const unsigned short&);
  friend int operator>=(const float_st&, const short&);
  friend int operator>=(const float_st&, const unsigned char&);
  friend int operator>=(const float_st&, const char&);

  friend int operator>=(const double&, const float_st&);
  friend int operator>=(const float&, const float_st&);
  friend int operator>=(const unsigned long long&, const float_st&);
  friend int operator>=(const long long&, const float_st&);
  friend int operator>=(const unsigned long&, const float_st&);
  friend int operator>=(const long&, const float_st&);
  friend int operator>=(const unsigned int&, const float_st&);
  friend int operator>=(const int&, const float_st&);
  friend int operator>=(const unsigned short&, const float_st&);
  friend int operator>=(const short&, const float_st&);
  friend int operator>=(const unsigned char&, const float_st&);
  friend int operator>=(const char&, const float_st&);

  friend int operator>(const double_st&, const float_st&);  
  friend int operator>(const float_st&, const double_st&);  
  friend int operator>(const float_st&, const float_st&);
  
  friend int operator>(const float_st&, const double&);
  friend int operator>(const float_st&, const float&);
  friend int operator>(const float_st&, const unsigned long long&);
  friend int operator>(const float_st&, const long long&);
  friend int operator>(const float_st&, const unsigned long&);
  friend int operator>(const float_st&, const long&);
  friend int operator>(const float_st&, const unsigned int&);
  friend int operator>(const float_st&, const int&);
  friend int operator>(const float_st&, const unsigned short&);
  friend int operator>(const float_st&, const short&);
  friend int operator>(const float_st&, const unsigned char&);
  friend int operator>(const float_st&, const char&);

  friend int operator>(const double&, const float_st&);
  friend int operator>(const float&, const float_st&);
  friend int operator>(const unsigned long long&, const float_st&);
  friend int operator>(const long long&, const float_st&);
  friend int operator>(const unsigned long&, const float_st&);
  friend int operator>(const long&, const float_st&);
  friend int operator>(const unsigned int&, const float_st&);
  friend int operator>(const int&, const float_st&);
  friend int operator>(const unsigned short&, const float_st&);
  friend int operator>(const short&, const float_st&);
  friend int operator>(const unsigned char&, const float_st&);
  friend int operator>(const char&, const float_st&);

  friend int operator<=(const double_st&, const float_st&);  
  friend int operator<=(const float_st&, const double_st&);  
  friend int operator<=(const float_st&, const float_st&);  

  friend int operator<=(const float_st&, const double&);
  friend int operator<=(const float_st&, const float&);
  friend int operator<=(const float_st&, const unsigned long long&);
  friend int operator<=(const float_st&, const long long&);
  friend int operator<=(const float_st&, const unsigned long&);
  friend int operator<=(const float_st&, const long&);
  friend int operator<=(const float_st&, const unsigned int&);
  friend int operator<=(const float_st&, const int&);
  friend int operator<=(const float_st&, const unsigned short&);
  friend int operator<=(const float_st&, const short&);
  friend int operator<=(const float_st&, const unsigned char&);
  friend int operator<=(const float_st&, const char&);

  friend int operator<=(const double&, const float_st&);
  friend int operator<=(const float&, const float_st&);
  friend int operator<=(const unsigned long long&, const float_st&);
  friend int operator<=(const long long&, const float_st&);
  friend int operator<=(const unsigned long&, const float_st&);
  friend int operator<=(const long&, const float_st&);
  friend int operator<=(const unsigned int&, const float_st&);
  friend int operator<=(const int&, const float_st&);
  friend int operator<=(const unsigned short&, const float_st&);
  friend int operator<=(const short&, const float_st&);
  friend int operator<=(const unsigned char&, const float_st&);
  friend int operator<=(const char&, const float_st&);

  friend int operator<(const double_st&, const float_st&);  
  friend int operator<(const float_st&, const double_st&);  
  friend int operator<(const float_st&, const float_st&);
  
  friend int operator<(const float_st&, const double&);
  friend int operator<(const float_st&, const float&);
  friend int operator<(const float_st&, const unsigned long long&);
  friend int operator<(const float_st&, const long long&);
  friend int operator<(const float_st&, const unsigned long&);
  friend int operator<(const float_st&, const long&);
  friend int operator<(const float_st&, const unsigned int&);
  friend int operator<(const float_st&, const int&);
  friend int operator<(const float_st&, const unsigned short&);
  friend int operator<(const float_st&, const short&);
  friend int operator<(const float_st&, const unsigned char&);
  friend int operator<(const float_st&, const char&);

  friend int operator<(const double&, const float_st&);
  friend int operator<(const float&, const float_st&);
  friend int operator<(const unsigned long long&, const float_st&);
  friend int operator<(const long long&, const float_st&);
  friend int operator<(const unsigned long&, const float_st&);
  friend int operator<(const long&, const float_st&);
  friend int operator<(const unsigned int&, const float_st&);
  friend int operator<(const int&, const float_st&);
  friend int operator<(const unsigned short&, const float_st&);
  friend int operator<(const short&, const float_st&);
  friend int operator<(const unsigned char&, const float_st&);
  friend int operator<(const char&, const float_st&);


  // MATHEMATICAL FUNCTIONS 

  friend float_st  logf(const float_st&);
  friend float_st  log2f(const float_st&);
  friend float_st  log10f(const float_st&);
  friend float_st  log1pf(const float_st&);
  friend float_st  logbf(const float_st&);
  friend float_st  expf(const float_st&); 
  friend float_st  exp2f(const float_st&); 
  friend float_st  expm1f(const float_st&); 
  friend float_st  sqrtf(const float_st&);
  friend float_st  cbrtf(const float_st&);
  friend float_st  sinf(const float_st&);
  friend float_st  cosf(const float_st&);
  friend float_st  tanf(const float_st&);
  friend float_st  asinf(const float_st&);
  friend float_st  acosf(const float_st&);
  friend float_st  atanf(const float_st&);
  friend double_st atan2f(const double_st&, const float_st&);
  friend double_st atan2f(const float_st&, const double_st&);
  friend float_st  atan2f(const float_st&, const float_st&);
  friend float_st  sinhf(const float_st&);
  friend float_st  coshf(const float_st&);
  friend float_st  tanhf(const float_st&);
  friend float_st  asinhf(const float_st&);
  friend float_st  acoshf(const float_st&);
  friend float_st  atanhf(const float_st&);
  friend double_st  hypot(const double_st&, const float_st&);
  friend double_st  hypot(const float_st&, const double_st&);  
  friend float_st  hypotf(const float_st&, const float_st&);

  friend float_st  log(const float_st&);
  friend float_st  log2(const float_st&);
  friend float_st  log10(const float_st&);
  friend float_st  log1p(const float_st&);
  friend float_st  logb(const float_st&);
  friend float_st  exp(const float_st&); 
  friend float_st  exp2(const float_st&); 
  friend float_st  expm1(const float_st&); 
  friend float_st  sqrt(const float_st&);
  friend float_st  cbrt(const float_st&);
  friend float_st  sin(const float_st&);
  friend float_st  cos(const float_st&);
  friend float_st  tan(const float_st&);
  friend float_st  asin(const float_st&);
  friend float_st  acos(const float_st&);
  friend float_st  atan(const float_st&);
  friend double_st atan2(const double_st&, const float_st&);
  friend double_st atan2(const float_st&, const double_st&);
  friend float_st  atan2(const float_st&, const float_st&);
  friend float_st  sinh(const float_st&);
  friend float_st  cosh(const float_st&);
  friend float_st  tanh(const float_st&);
  friend float_st  asinh(const float_st&);
  friend float_st  acosh(const float_st&);
  friend float_st  atanh(const float_st&);
  // friend double_st  hypot(const double_st&, const float_st&);
  // friend double_st  hypot(const float_st&, const double_st&);  
  friend float_st  hypot(const float_st&, const float_st&);



  friend double_st pow(const double_st&, const float_st&);
  friend double_st pow(const float_st&, const double_st&);
  friend float_st powf(const float_st&, const float_st&);
  friend float_st pow(const float_st&, const float_st&);
  
  friend float_st powf(const float_st&, const double&);
  friend float_st powf(const float_st&, const float&);
  friend float_st powf(const float_st&, const unsigned long long&);
  friend float_st powf(const float_st&, const long long&);
  friend float_st powf(const float_st&, const unsigned long&);
  friend float_st powf(const float_st&, const long&);
  friend float_st powf(const float_st&, const unsigned int&);
  friend float_st powf(const float_st&, const int&);
  friend float_st powf(const float_st&, const unsigned short&);
  friend float_st powf(const float_st&, const short&);
  friend float_st powf(const float_st&, const unsigned char&);
  friend float_st powf(const float_st&, const char&);

  friend float_st pow(const float_st&, const double&);
  friend float_st pow(const float_st&, const float&);
  friend float_st pow(const float_st&, const unsigned long long&);
  friend float_st pow(const float_st&, const long long&);
  friend float_st pow(const float_st&, const unsigned long&);
  friend float_st pow(const float_st&, const long&);
  friend float_st pow(const float_st&, const unsigned int&);
  friend float_st pow(const float_st&, const int&);
  friend float_st pow(const float_st&, const unsigned short&);
  friend float_st pow(const float_st&, const short&);
  friend float_st pow(const float_st&, const unsigned char&);
  friend float_st pow(const float_st&, const char&);



  friend float_st powf(const double&, const float_st&);
  friend float_st powf(const float&, const float_st&);
  friend float_st powf(const unsigned long long&, const float_st&);
  friend float_st powf(const long long&, const float_st&);
  friend float_st powf(const unsigned long&, const float_st&);
  friend float_st powf(const long&, const float_st&);
  friend float_st powf(const unsigned int&, const float_st&);
  friend float_st powf(const int&, const float_st&);
  friend float_st powf(const unsigned short&, const float_st&);
  friend float_st powf(const short&, const float_st&);
  friend float_st powf(const unsigned char&, const float_st&);
  friend float_st powf(const char&, const float_st&);

  friend float_st pow(const double&, const float_st&);
  friend float_st pow(const float&, const float_st&);
  friend float_st pow(const unsigned long long&, const float_st&);
  friend float_st pow(const long long&, const float_st&);
  friend float_st pow(const unsigned long&, const float_st&);
  friend float_st pow(const long&, const float_st&);
  friend float_st pow(const unsigned int&, const float_st&);
  friend float_st pow(const int&, const float_st&);
  friend float_st pow(const unsigned short&, const float_st&);
  friend float_st pow(const short&, const float_st&);
  friend float_st pow(const unsigned char&, const float_st&);
  friend float_st pow(const char&, const float_st&);


  friend float_st fmaxf(const float_st&, const float_st&);  
  friend double_st fmax(const double_st&, const float_st&);  
  friend double_st fmax(const float_st&, const double_st&);  

  friend float_st fmax(const float_st&, const float_st&);  

  friend float_st fmaxf(const float_st&, const double&);
  friend float_st fmaxf(const float_st&, const float&);
  friend float_st fmaxf(const float_st&, const unsigned long long&);
  friend float_st fmaxf(const float_st&, const long long&);
  friend float_st fmaxf(const float_st&, const unsigned long&);
  friend float_st fmaxf(const float_st&, const long&);
  friend float_st fmaxf(const float_st&, const unsigned int&);
  friend float_st fmaxf(const float_st&, const int&);
  friend float_st fmaxf(const float_st&, const unsigned short&);
  friend float_st fmaxf(const float_st&, const short&);
  friend float_st fmaxf(const float_st&, const unsigned char&);
  friend float_st fmaxf(const float_st&, const char&);

  friend float_st fmax(const float_st&, const double&);
  friend float_st fmax(const float_st&, const float&);
  friend float_st fmax(const float_st&, const unsigned long long&);
  friend float_st fmax(const float_st&, const long long&);
  friend float_st fmax(const float_st&, const unsigned long&);
  friend float_st fmax(const float_st&, const long&);
  friend float_st fmax(const float_st&, const unsigned int&);
  friend float_st fmax(const float_st&, const int&);
  friend float_st fmax(const float_st&, const unsigned short&);
  friend float_st fmax(const float_st&, const short&);
  friend float_st fmax(const float_st&, const unsigned char&);
  friend float_st fmax(const float_st&, const char&);


  friend float_st fmaxf(const double&, const float_st&);
  friend float_st fmaxf(const float&, const float_st&);
  friend float_st fmaxf(const unsigned long long&, const float_st&);
  friend float_st fmaxf(const long long&, const float_st&);
  friend float_st fmaxf(const unsigned long&, const float_st&);
  friend float_st fmaxf(const long&, const float_st&);
  friend float_st fmaxf(const unsigned int&, const float_st&);
  friend float_st fmaxf(const int&, const float_st&);
  friend float_st fmaxf(const unsigned short&, const float_st&);
  friend float_st fmaxf(const short&, const float_st&);
  friend float_st fmaxf(const unsigned char&, const float_st&);
  friend float_st fmaxf(const char&, const float_st&);

  friend float_st fmax(const double&, const float_st&);
  friend float_st fmax(const float&, const float_st&);
  friend float_st fmax(const unsigned long long&, const float_st&);
  friend float_st fmax(const long long&, const float_st&);
  friend float_st fmax(const unsigned long&, const float_st&);
  friend float_st fmax(const long&, const float_st&);
  friend float_st fmax(const unsigned int&, const float_st&);
  friend float_st fmax(const int&, const float_st&);
  friend float_st fmax(const unsigned short&, const float_st&);
  friend float_st fmax(const short&, const float_st&);
  friend float_st fmax(const unsigned char&, const float_st&);
  friend float_st fmax(const char&, const float_st&);


  friend float_st fminf(const float_st&, const float_st&);  
  friend float_st fmin(const float_st&, const float_st&);  
  friend double_st fmin(const double_st&, const float_st&);  
  friend double_st fmin(const float_st&, const double_st&);  

  friend float_st fminf(const float_st&, const double&);
  friend float_st fminf(const float_st&, const float&);
  friend float_st fminf(const float_st&, const unsigned long long&);
  friend float_st fminf(const float_st&, const long long&);
  friend float_st fminf(const float_st&, const unsigned long&);
  friend float_st fminf(const float_st&, const long&);
  friend float_st fminf(const float_st&, const unsigned int&);
  friend float_st fminf(const float_st&, const int&);
  friend float_st fminf(const float_st&, const unsigned short&);
  friend float_st fminf(const float_st&, const short&);
  friend float_st fminf(const float_st&, const unsigned char&);
  friend float_st fminf(const float_st&, const char&);

  friend float_st fmin(const float_st&, const double&);
  friend float_st fmin(const float_st&, const float&);
  friend float_st fmin(const float_st&, const unsigned long long&);
  friend float_st fmin(const float_st&, const long long&);
  friend float_st fmin(const float_st&, const unsigned long&);
  friend float_st fmin(const float_st&, const long&);
  friend float_st fmin(const float_st&, const unsigned int&);
  friend float_st fmin(const float_st&, const int&);
  friend float_st fmin(const float_st&, const unsigned short&);
  friend float_st fmin(const float_st&, const short&);
  friend float_st fmin(const float_st&, const unsigned char&);
  friend float_st fmin(const float_st&, const char&);


  friend float_st fminf(const double&, const float_st&);
  friend float_st fminf(const float&, const float_st&);
  friend float_st fminf(const unsigned long long&, const float_st&);
  friend float_st fminf(const long long&, const float_st&);
  friend float_st fminf(const unsigned long&, const float_st&);
  friend float_st fminf(const long&, const float_st&);
  friend float_st fminf(const unsigned int&, const float_st&);
  friend float_st fminf(const int&, const float_st&);
  friend float_st fminf(const unsigned short&, const float_st&);
  friend float_st fminf(const short&, const float_st&);
  friend float_st fminf(const unsigned char&, const float_st&);
  friend float_st fminf(const char&, const float_st&);

  friend float_st fmin(const double&, const float_st&);
  friend float_st fmin(const float&, const float_st&);
  friend float_st fmin(const unsigned long long&, const float_st&);
  friend float_st fmin(const long long&, const float_st&);
  friend float_st fmin(const unsigned long&, const float_st&);
  friend float_st fmin(const long&, const float_st&);
  friend float_st fmin(const unsigned int&, const float_st&);
  friend float_st fmin(const int&, const float_st&);
  friend float_st fmin(const unsigned short&, const float_st&);
  friend float_st fmin(const short&, const float_st&);
  friend float_st fmin(const unsigned char&, const float_st&);
  friend float_st fmin(const char&, const float_st&);

  // INTRINSIC FUNCTIONS
 
  friend float_st fabsf(const float_st&);
  friend float_st floorf(const float_st&);
  friend float_st ceilf(const float_st&);
  friend float_st truncf(const float_st&);
  friend float_st nearbyintf(const float_st&);
  friend float_st rintf(const float_st&);
  friend long int  lrintf(const float_st&);
  friend long long int  llrintf(const float_st&);

  friend float_st frexp(const float_st&, int*);
  friend float_st modf(const  float_st&,  float_st*);
  friend float_st ldexp(const float_st&, const int);
  friend float_st fmod(const float_st&, const float_st&);


  friend float_st frexpf(const float_st&, int*);
  friend float_st modff(const  float_st&,  float_st*);
  friend float_st ldexpf(const float_st&, const int);
  friend float_st fmodf(const float_st&, const float_st&);


  friend float_st fabs(const float_st&);
  friend float_st abs(const float_st&);
  friend float_st floor(const float_st&);
  friend float_st ceil(const float_st&);
  friend float_st trunc(const float_st&);
  friend float_st nearbyint(const float_st&);
  friend float_st rint(const float_st&);
  friend long int  lrint(const float_st&);
  friend long long int  llrint(const float_st&);

  friend int finite(const float_st&);
  friend int isfinite(const float_st&);
  friend int isnan(const float_st&);
  friend int isinf(const float_st&);


  // Conversion
  operator char();  
  operator unsigned char();  
  operator short();  
  operator unsigned short();  
  operator int();  
  operator unsigned();  
  operator long();  
  operator unsigned long();  
  operator long long();  
  operator unsigned long long();  
  operator float();
  operator double();
  operator double_st();
  



  
  int nb_significant_digit() const;  
  int approx_digit() const;  
  int computedzero() const;  
  int numericalnoise() const;  
  void display() const ; 
  void display(const char *) const ; 
  char* str( char *)  const ;
  
  friend char* strp(const float_st&);
  friend char* str( char *, const float_st&);
  
  friend std::istream& operator >>(std::istream& s, float_st& );  
  
  void data_st();
  void data_st(const double&, const int &);
  
};

std::ostream& operator<<(std::ostream&, const float_st&);
std::istream& operator >>(std::istream&, float_st& );



inline double_st::double_st (const float_st&a) : x(a.x),y(a.y),z(a.z),accuracy(a.accuracy) {};


void cadna_set_rnd_arr();
void cadna_set_rnd_plinf();
void cadna_set_rnd_moinf();
void cadna_set_rnd_zero();

void  cadna_init(int, unsigned int, unsigned int, unsigned int);
void  cadna_init(int, unsigned int);
void  cadna_init(int);
void  cadna_end();

void cadna_enable(unsigned int);
void cadna_disable(unsigned int);

#endif











