//Copyright 2016 J.-M. CHESNEAUX, F. JEZEQUEL, J.-L. LAMOTTE
//
//This file is part of CADNA.
//
//    CADNA is free software: you can redistribute it and/or modify it
//    under the terms of the GNU Lesser General Public License as
//    published by the Free Software Foundation, either version 3 of the
//    License, or (at your option) any later version.
//
//    CADNA is distributed in the hope that it will be useful, but WITHOUT
//    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
//    Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with CADNA. If not, see
//    <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////
//****m* cadna_ge/operator>=
//    NAME
//      operator>=
//    SYNOPSIS
//      res = a >= b 
//    FUNCTION
//      Define all the functions involving at least one argument
//      of stochastic type which overload the ">=" operator
//      in a test such as "a>=b".
//      
//    INPUTS
//      a           - an integer, a float, a double or a stochastic number
//      b           - an integer, a float, a double or a stochastic number 
//      At least one argument must be of stochastic type.
//    RESULT
//      res         - an integer value
//             
//*****
//   You can use this space for remarks that should not be included
//   in the documentation.
//    EXAMPLE
//      
//  
//    NOTES
//    The output value takes into account the stochastic definition of
//    the ">=" operator (see the documentation).
//  
//    BUGS
//  
//  
//    SEE ALSO
//      
//      
//  /

#include "cadna.h"
#include "cadna_private.h"


int operator>=(const double& a, const double_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const float& a, const double_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const unsigned long long& a, const double_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const long long& a, const double_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const unsigned long& a, const double_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const long& a, const double_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const unsigned int & a, const double_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const int & a, const double_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const unsigned short& a, const double_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const short& a, const double_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const unsigned char& a, const double_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const char& a, const double_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 

int operator>=(const double& a, const float_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const float& a, const float_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const unsigned long long& a, const float_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const long long& a, const float_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const unsigned long& a, const float_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const long& a, const float_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const unsigned int & a, const float_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const int & a, const float_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const unsigned short& a, const float_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const short& a, const float_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const unsigned char& a, const float_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 
int operator>=(const char& a, const float_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a-b.x; 
  if (RANDOM) rnd_switch(); res.y=a-b.y; 
  rnd_switch();   res.z=a-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( 3*a ) >=(  b.x + b.y + b.z )); 
}
 
 

///////////////////////////////////////////////////




int operator>=(const double_st& a, const  double& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const double_st& a, const  float& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const double_st& a, const  unsigned long long& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const double_st& a, const  long long& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const double_st& a, const  unsigned long& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const double_st& a, const  long& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const double_st& a, const  unsigned int & b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const double_st& a, const  int & b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const double_st& a, const  unsigned short& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const double_st& a, const  short& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const double_st& a, const  unsigned char& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const double_st& a, const  char& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 

int operator>=(const float_st& a, const  double& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const float_st& a, const  float& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const float_st& a, const  unsigned long long& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const float_st& a, const  long long& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const float_st& a, const  unsigned long& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const float_st& a, const  long& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const float_st& a, const  unsigned int & b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const float_st& a, const  int & b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const float_st& a, const  unsigned short& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const float_st& a, const  short& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const float_st& a, const  unsigned char& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 
int operator>=(const float_st& a, const  char& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b; 
  if (RANDOM) rnd_switch(); res.y=a.y-b; 
  rnd_switch(); res.z=a.z-b; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || (  (a.x + a.y + a.z ) >= (3*b )); 
}
 
 

///////////////////////////////////////////////////


int operator>=(const double_st& a, const double_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b.x; 
  if (RANDOM) rnd_switch(); res.y=a.y-b.y; 
  rnd_switch(); res.z=a.z-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( a.x + a.y + a.z ) >=	( b.x + b.y + b.z )); 
}
 
 
int operator>=(const double_st& a, const float_st& b) 
{ 
  double_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b.x; 
  if (RANDOM) rnd_switch(); res.y=a.y-b.y; 
  rnd_switch(); res.z=a.z-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( a.x + a.y + a.z ) >=	( b.x + b.y + b.z )); 
}
 
 
int operator>=(const float_st& a, const double_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b.x; 
  if (RANDOM) rnd_switch(); res.y=a.y-b.y; 
  rnd_switch(); res.z=a.z-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( a.x + a.y + a.z ) >=	( b.x + b.y + b.z )); 
}
 
 
int operator>=(const float_st& a, const float_st& b) 
{ 
  float_st res; 
   
  if (RANDOM) rnd_switch(); res.x=a.x-b.x; 
  if (RANDOM) rnd_switch(); res.y=a.y-b.y; 
  rnd_switch(); res.z=a.z-b.z; 
   
  int r=res.numericalnoise(); 
  if (_cadna_branching_tag && r) instability(&_cadna_branching_count); 
  return r || ( ( a.x + a.y + a.z ) >=	( b.x + b.y + b.z )); 
}
 
 

