//Copyright 2016 J.-M. CHESNEAUX, F. JEZEQUEL, J.-L. LAMOTTE
//
//This file is part of CADNA.
//
//    CADNA is free software: you can redistribute it and/or modify it
//    under the terms of the GNU Lesser General Public License as
//    published by the Free Software Foundation, either version 3 of the
//    License, or (at your option) any later version.
//
//    CADNA is distributed in the hope that it will be useful, but WITHOUT
//    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
//    Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with CADNA. If not, see
//    <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////
//****m* cadna_mul/operator*
//    NAME
//      operator*
//    SYNOPSIS
//      res = a * b 
//    FUNCTION
//      Defines all the functions involving at least one argument
//      of stochastic type which overload the "*" operator
//      in a statement such as "a*b".
//      
//    INPUTS
//      a           - an integer, a float, a double or a stochastic number
//      b           - an integer, a float, a double or a stochastic number 
//      At least one argument must be of stochastic type.
//    RESULT
//      res         - a stochastic number
//             
//*****
//   You can use this space for remarks that should not be included
//   in the documentation.
//    EXAMPLE
//      
//  
//    NOTES
//  
//  
//    BUGS
//  
//  
//    SEE ALSO
//      
//      
//  /

#include "cadna.h"
#include "cadna_private.h"


/////////////////////////////////////////////////////////////////////////



float_st& float_st::operator*=(const double& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
float_st& float_st::operator*=(const float& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
float_st& float_st::operator*=(const unsigned long long& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
float_st& float_st::operator*=(const long long& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
float_st& float_st::operator*=(const unsigned long& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
float_st& float_st::operator*=(const long& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
float_st& float_st::operator*=(const unsigned int& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
float_st& float_st::operator*=(const int& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
float_st& float_st::operator*=(const unsigned short& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
float_st& float_st::operator*=(const short& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
float_st& float_st::operator*=(const unsigned char& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
float_st& float_st::operator*=(const char& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 

double_st& double_st::operator*=(const double& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
double_st& double_st::operator*=(const float& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
double_st& double_st::operator*=(const unsigned long long& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
double_st& double_st::operator*=(const long long& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
double_st& double_st::operator*=(const unsigned long& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
double_st& double_st::operator*=(const long& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
double_st& double_st::operator*=(const unsigned int& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
double_st& double_st::operator*=(const int& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
double_st& double_st::operator*=(const unsigned short& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
double_st& double_st::operator*=(const short& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
double_st& double_st::operator*=(const unsigned char& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 
double_st& double_st::operator*=(const char& a)  
{ 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch(); 
  x*=a; 
  if (RANDOM) rnd_switch(); 
  y*=a; 
  rnd_switch(); 
  z*=a; 
  return *this; 
}
 
 

/////////////////////////////////////////////////////////////////////////


double_st& double_st::operator*=(const  double_st& a)	 
{ 
  if(_cadna_mul_tag) { 
    if (accuracy==DIGIT_NOT_COMPUTED) approx_digit();	 
    if (accuracy==0){ 
      if (a.accuracy==DIGIT_NOT_COMPUTED) a.approx_digit(); 
      if (a.accuracy==0) 
	instability(&_cadna_mul_count); 
    } 
  } 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  x*=a.x; 
  if (RANDOM) rnd_switch();   
  y*=a.y; 
  rnd_switch();   
  z*=a.z; 
  return *this; 
}
 
 
double_st& double_st::operator*=(const  float_st& a)	 
{ 
  if(_cadna_mul_tag) { 
    if (accuracy==DIGIT_NOT_COMPUTED) approx_digit();	 
    if (accuracy==0){ 
      if (a.accuracy==DIGIT_NOT_COMPUTED) a.approx_digit(); 
      if (a.accuracy==0) 
	instability(&_cadna_mul_count); 
    } 
  } 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  x*=a.x; 
  if (RANDOM) rnd_switch();   
  y*=a.y; 
  rnd_switch();   
  z*=a.z; 
  return *this; 
}
 
 
float_st& float_st::operator*=(const  double_st& a)	 
{ 
  if(_cadna_mul_tag) { 
    if (accuracy==DIGIT_NOT_COMPUTED) approx_digit();	 
    if (accuracy==0){ 
      if (a.accuracy==DIGIT_NOT_COMPUTED) a.approx_digit(); 
      if (a.accuracy==0) 
	instability(&_cadna_mul_count); 
    } 
  } 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  x*=a.x; 
  if (RANDOM) rnd_switch();   
  y*=a.y; 
  rnd_switch();   
  z*=a.z; 
  return *this; 
}
 
 
float_st& float_st::operator*=(const  float_st& a)	 
{ 
  if(_cadna_mul_tag) { 
    if (accuracy==DIGIT_NOT_COMPUTED) approx_digit();	 
    if (accuracy==0){ 
      if (a.accuracy==DIGIT_NOT_COMPUTED) a.approx_digit(); 
      if (a.accuracy==0) 
	instability(&_cadna_mul_count); 
    } 
  } 
  accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  x*=a.x; 
  if (RANDOM) rnd_switch();   
  y*=a.y; 
  rnd_switch();   
  z*=a.z; 
  return *this; 
}
 
 

/////////////////////////////////////////////////////////////////////////


double_st operator*(const  double& a, const double_st& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
double_st operator*(const  float& a, const double_st& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
double_st operator*(const  unsigned long long& a, const double_st& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
double_st operator*(const  long long& a, const double_st& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
double_st operator*(const  unsigned long& a, const double_st& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
double_st operator*(const  long& a, const double_st& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
double_st operator*(const  unsigned int & a, const double_st& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
double_st operator*(const  int & a, const double_st& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
double_st operator*(const  unsigned short& a, const double_st& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
double_st operator*(const  short& a, const double_st& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
double_st operator*(const  unsigned char& a, const double_st& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
double_st operator*(const  char& a, const double_st& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 

float_st operator*(const  double& a, const float_st& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
float_st operator*(const  float& a, const float_st& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
float_st operator*(const  unsigned long long& a, const float_st& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
float_st operator*(const  long long& a, const float_st& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
float_st operator*(const  unsigned long& a, const float_st& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
float_st operator*(const  long& a, const float_st& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
float_st operator*(const  unsigned int & a, const float_st& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
float_st operator*(const  int & a, const float_st& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
float_st operator*(const  unsigned short& a, const float_st& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
float_st operator*(const  short& a, const float_st& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
float_st operator*(const  unsigned char& a, const float_st& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 
float_st operator*(const  char& a, const float_st& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a*b.y; 
  rnd_switch();   
  res.z=a*b.z; 
  return res ; 
}
 
 

/////////////////////////////////////////////////////////////////////////


double_st operator*(const  double_st& a, const  double& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
double_st operator*(const  double_st& a, const  float& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
double_st operator*(const  double_st& a, const  unsigned long long& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
double_st operator*(const  double_st& a, const  long long& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
double_st operator*(const  double_st& a, const  unsigned long& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
double_st operator*(const  double_st& a, const  long& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
double_st operator*(const  double_st& a, const  unsigned int & b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
double_st operator*(const  double_st& a, const  int & b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
double_st operator*(const  double_st& a, const  unsigned short& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
double_st operator*(const  double_st& a, const  short& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
double_st operator*(const  double_st& a, const  unsigned char& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
double_st operator*(const  double_st& a, const  char& b) 
{ 
  double_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 

float_st operator*(const  float_st& a, const  double& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
float_st operator*(const  float_st& a, const  float& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
float_st operator*(const  float_st& a, const  unsigned long long& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
float_st operator*(const  float_st& a, const  long long& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
float_st operator*(const  float_st& a, const  unsigned long& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
float_st operator*(const  float_st& a, const  long& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
float_st operator*(const  float_st& a, const  unsigned int & b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
float_st operator*(const  float_st& a, const  int & b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
float_st operator*(const  float_st& a, const  unsigned short& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
float_st operator*(const  float_st& a, const  short& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
float_st operator*(const  float_st& a, const  unsigned char& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 
float_st operator*(const  float_st& a, const  char& b) 
{ 
  float_st res; 
 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b; 
  rnd_switch();   
  res.z=a.z*b; 
  return res ; 
}
 
 

/////////////////////////////////////////////////////////////////////////


double_st operator*(const double_st& a, const double_st& b) 
{ 
  double_st res; 
  if(_cadna_mul_tag) { 
    if (a.accuracy==DIGIT_NOT_COMPUTED) a.approx_digit();	 
    if (a.accuracy==0){ 
      if (b.accuracy==DIGIT_NOT_COMPUTED) b.approx_digit(); 
      if (b.accuracy==0) 
	instability(&_cadna_mul_count); 
    } 
  } 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b.y; 
  rnd_switch();   
  res.z=a.z*b.z; 
  return res ; 
}
 
 
double_st operator*(const double_st& a, const float_st& b) 
{ 
  double_st res; 
  if(_cadna_mul_tag) { 
    if (a.accuracy==DIGIT_NOT_COMPUTED) a.approx_digit();	 
    if (a.accuracy==0){ 
      if (b.accuracy==DIGIT_NOT_COMPUTED) b.approx_digit(); 
      if (b.accuracy==0) 
	instability(&_cadna_mul_count); 
    } 
  } 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b.y; 
  rnd_switch();   
  res.z=a.z*b.z; 
  return res ; 
}
 
 
double_st operator*(const float_st& a, const double_st& b) 
{ 
  double_st res; 
  if(_cadna_mul_tag) { 
    if (a.accuracy==DIGIT_NOT_COMPUTED) a.approx_digit();	 
    if (a.accuracy==0){ 
      if (b.accuracy==DIGIT_NOT_COMPUTED) b.approx_digit(); 
      if (b.accuracy==0) 
	instability(&_cadna_mul_count); 
    } 
  } 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b.y; 
  rnd_switch();   
  res.z=a.z*b.z; 
  return res ; 
}
 
 
float_st operator*(const float_st& a, const float_st& b) 
{ 
  float_st res; 
  if(_cadna_mul_tag) { 
    if (a.accuracy==DIGIT_NOT_COMPUTED) a.approx_digit();	 
    if (a.accuracy==0){ 
      if (b.accuracy==DIGIT_NOT_COMPUTED) b.approx_digit(); 
      if (b.accuracy==0) 
	instability(&_cadna_mul_count); 
    } 
  } 
  res.accuracy=DIGIT_NOT_COMPUTED; 
  if (RANDOM) rnd_switch();   
  res.x=a.x*b.x; 
  if (RANDOM) rnd_switch();   
  res.y=a.y*b.y; 
  rnd_switch();   
  res.z=a.z*b.z; 
  return res ; 
}
 
 

/////////////////////////////////////////////////////////////////////////

